@@ -2,7 +2,8 @@ from logging import getLogger
 from os import path, getpid
 from infi.pyutils.contexts import contextmanager
 
-from .utils import func_logger, check_for_scsi_errors, asi_context, log_execute, ScsiCommandFailed, TIMEOUT_IN_SEC
+from .utils import func_logger, check_for_scsi_errors, asi_context, log_execute, TIMEOUT_IN_SEC
+from .utils import ScsiCommandFailed, ScsiCheckConditionError
 
 logger = getLogger(__name__)
 
@@ -127,6 +128,7 @@ def read_from_queue(reader, subprocess):
         except:
             msg = "{} multiprocessing {} error"
             logger.exception(msg.format(getpid(), subprocess.pid))
+            return ScsiCommandFailed()
 
 def ensure_subprocess_dead(subprocess):
     if subprocess.is_alive():
@@ -147,6 +149,8 @@ def do_scsi_cdb(sg_device, cdb):
         return_value = read_from_queue(reader, subprocess)
         logger.debug("{} multiprocessing {} returned {!r}".format(getpid(), subprocess.pid, return_value))
         ensure_subprocess_dead(subprocess)
+    if isinstance(return_value, ScsiCheckConditionError):
+        raise ScsiCheckConditionError(return_value.sense_key, return_value.code_name)
     if isinstance(return_value, ScsiCommandFailed):
         raise ScsiCommandFailed()
     return return_value
@@ -160,7 +164,15 @@ def do_report_luns(sg_device):
 @func_logger
 def do_test_unit_ready(sg_device):
     from infi.asi.cdb.tur import TestUnitReadyCommand
-    cdb = TestUnitReadyCommand()
+    try:
+        cdb = TestUnitReadyCommand()
+    except ScsiCheckConditionError, error:
+        (key, code) = (error.sense_key, error.code_name)
+        if (key, code) == ('NOT_READY', 'MEDIUM NOT PRESENT'):
+            return False
+        if (key, code) == ('ILLEGAL_REQUEST', 'INVALID COMMAND OPERATION CODE'):
+            return False
+        raise
     return do_scsi_cdb(sg_device, cdb)
 
 @func_logger
